@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 partially based on C++ code from:
     Copyright (C) 2006 Mauricio Piacentini <mauricio@tabuleiro.com>
@@ -98,10 +98,10 @@ class Preferences(KConfigSkeleton):
         self.addString('General', 'tilesetName', 'default')
         self.addString('General', 'windTilesetName', 'traditional')
         self.addString('General', 'backgroundName', 'wood_light')
-        self.addInteger('Network', 'serverPort', 8149)
         self.addBool('Display', 'showShadows', True)
         self.addBool('Display', 'rearrangeMelds', False)
         self.addBool('Display', 'showOnlyPossibleActions', True)
+        self.addBool('Display', 'propose', True)
         self.addInteger('Display', 'animationSpeed', 70, 0, 99)
         self.addBool('Display', 'useSounds', True)
         self.addBool('Display', 'uploadVoice', False)
