@@ -31,15 +31,15 @@ from kde import KApplication
 
 from genericdelegates import RichTextColumnDelegate
 
-from util import logWarning, m18n, m18nc, logDebug
+from util import logWarning, m18n, m18nc, m18nE, logDebug
 from statesaver import StateSaver
 from rule import Ruleset
 from guiutil import ListComboBox, MJTableView
 from differ import RulesetDiffer
 from common import InternalParameters, Debug
 from client import ClientTable
-from chat import ChatWindow
 from modeltest import ModelTest
+from chat import ChatMessage, ChatWindow
 
 class TablesModel(QAbstractTableModel):
     """a model for our tables"""
@@ -160,6 +160,7 @@ class TableList(QWidget):
         self.view = MJTableView(self)
         self.differ = None
         self.debugModelTest = None
+        self.__requestedNewTable = False
         self.view.setItemDelegateForColumn(2, RichTextColumnDelegate(self.view))
 
         buttonBox = QDialogButtonBox(self)
@@ -199,9 +200,28 @@ class TableList(QWidget):
         StateSaver(self, self.view.horizontalHeader())
         self.updateButtonsForTable(None)
 
+    def hideEvent(self, dummyEvent): # pylint: disable=R0201
+        """table window hides"""
+        InternalParameters.field.startingGame = False
+
     def chat(self):
-        """chat"""
-        ChatWindow.createFor(self.selectedTable())
+        """chat. Only generate ChatWindow after the
+        message has successfully been sent to the server.
+        Because the server might have gone away."""
+        def initChat(_):
+            """now that we were able to send the message to the server
+            instantiate the chat window"""
+            table.chatWindow = ChatWindow(table)
+            table.chatWindow.receiveLine(msg)
+        table = self.selectedTable()
+        if not table.chatWindow:
+            line = m18nE('opens a chat window')
+            msg = ChatMessage(table.tableid, table.client.username, line, isStatusMessage=True)
+            table.client.sendChat(msg).addCallback(initChat).addErrback(self.tableError)
+        elif table.chatWindow.isVisible():
+            table.chatWindow.hide()
+        else:
+            table.chatWindow.show()
 
     def show(self):
         """prepare the view and show it"""
@@ -242,8 +262,9 @@ class TableList(QWidget):
             not running and
             not table.isOnline(self.client.username) and
             (self.client.username in table.playerNames) == suspended)
-        for btn in [self.leaveButton, self.startButton, self.compareButton]:
-            btn.setVisible(not (suspendedLocalGame))
+        self.leaveButton.setVisible(not (suspendedLocalGame))
+        self.compareButton.setVisible(not (suspendedLocalGame))
+        self.startButton.setVisible(not suspended)
         if suspendedLocalGame:
             self.newButton.setToolTip(m18n("Start a new game"))
             self.joinButton.setText(m18nc('resuming a local suspended game', '&Resume'))
@@ -287,9 +308,10 @@ class TableList(QWidget):
                 return
             ruleset = selectDialog.cbRuleset.current
         deferred = self.client.callServer('newTable', ruleset.toList(),
-            InternalParameters.playOpen, InternalParameters.autoPlay, self.__wantedGame())
+            InternalParameters.playOpen, InternalParameters.autoPlay, self.__wantedGame()).addErrback(self.tableError)
         if self.client.hasLocalServer():
             deferred.addCallback(self.newLocalTable)
+        self.__requestedNewTable = True
 
     def gotTables(self, tables):
         """got tables for first time. If we play a local game and we have no
@@ -303,7 +325,7 @@ class TableList(QWidget):
         if InternalParameters.autoPlay or (not clientTables and self.client.hasLocalServer()):
             deferred = self.client.callServer('newTable', self.client.ruleset.toList(), InternalParameters.playOpen,
                 InternalParameters.autoPlay,
-                self.__wantedGame())
+                self.__wantedGame()).addErrback(self.tableError)
             if deferred:
                 deferred.addCallback(self.newLocalTable)
         else:
@@ -324,6 +346,7 @@ class TableList(QWidget):
         if len(table.humanPlayerNames()) - 1 == sum(table.playersOnline):
             # we are the last human player joining, so the server will start the game
             self.hide()
+        self.__requestedNewTable = table.status.startswith('Suspended') # because tableid will change
         self.client.callServer('joinTable', table.tableid).addErrback(self.tableError)
 
     def compareRuleset(self):
@@ -338,14 +361,24 @@ class TableList(QWidget):
         self.startButton.setEnabled(False)
         self.client.callServer('startGame', table.tableid).addErrback(self.tableError)
 
-    @staticmethod
-    def tableError(err):
+    def tableError(self, err):
         """log the twisted error"""
-        logWarning(err.getErrorMessage())
+        if not self.client.perspective:
+            # lost connection to server
+            for table in self.view.model().tables:
+                if table.chatWindow:
+                    table.chatWindow.hide()
+            self.hide()
+            self.client.tableList = None
+        else:
+            logWarning(err.getErrorMessage())
 
     def leaveTable(self):
         """leave a table"""
-        self.client.callServer('leaveTable', self.selectedTable().tableid)
+        table = self.selectedTable()
+        self.__requestedNewTable = True
+        self.__requestedNewTable = table.status.startswith('Suspended') # because tableid will change
+        self.client.callServer('leaveTable', table.tableid).addErrback(self.tableError)
 
     def __keepChatWindows(self, tables):
         """copy chatWindows from the old table list which will be thrown away"""
@@ -359,6 +392,25 @@ class TableList(QWidget):
                 if unusedWindow:
                     unusedWindow.hide()
 
+    def __preselectTableId(self, tables):
+        """which table should be preselected?
+        If we just requested a new table:
+          select first new table.
+          Only in the rare case that two clients request a new table at the same
+          moment, this could put the focus on the wrong table. Ignore that for now.
+        else if we had one selected:
+          select that again
+        else:
+          select first table"""
+        if self.__requestedNewTable:
+            self.__requestedNewTable = False
+            newIds = sorted(list(set(x.tableid for x in tables) - set(x.tableid for x in self.view.model().tables)))
+            if newIds:
+                return newIds[0]
+        if self.selectedTable():
+            return self.selectedTable().tableid
+        return 0
+
     def loadTables(self, tables):
         """build and use a model around the tables.
         Show all new tables (no gameid given yet) and all suspended
@@ -374,6 +426,8 @@ class TableList(QWidget):
                 elif not table.gameExistsLocally():
                     logDebug('%s does not exist locally' % table)
         tables = [x for x in tables if not x.gameid or x.gameExistsLocally()]
+        tables.sort(key=lambda x: x.tableid)
+        preselectTableId = self.__preselectTableId(tables)
         self.__keepChatWindows(tables)
         model = TablesModel(tables)
         self.view.setModel(model)
@@ -386,10 +440,12 @@ class TableList(QWidget):
         self.view.setSelectionMode(QAbstractItemView.SingleSelection)
         selection.selectionChanged.connect(self.selectionChanged)
         if len(tables) == 1:
+            self.selectTable(0)
             self.startButton.setFocus()
         elif not tables:
             self.newButton.setFocus()
-        if not self.selectedTable() and self.view.model().rowCount():
-            self.selectTable(0)
+        else:
+            _ = [x for x in tables if x.tableid >= preselectTableId]
+            self.selectTable(tables.index(_[0]))
         self.updateButtonsForTable(self.selectedTable())
         self.view.setFocus()
