@@ -10,7 +10,8 @@
 #ifndef PLAYERLABEL_H
 #define PLAYERLABEL_H
 
-#include <KGameCanvas>
+#define USE_UNSTABLE_LIBKDEGAMESPRIVATE_API
+#include <libkdegamesprivate/kgamecanvas.h>
 
 class PlayerLabel : public KGameCanvasGroup
 {
