@@ -64,11 +64,14 @@ KRightDialog::KRightDialog(QDockWidget * parent, ONU * world,KGameWindow* m_game
     KConfig config(world->getConfigFileName());
     KConfigGroup onugroup = config.group("onu");
     QString skin = onugroup.readEntry("skinpath");
-    QString imageFileName = KGlobal::dirs()->findResource("appdata", skin + "/Images/sprites/infantry.svg");
-    soldat.load(imageFileName);
 
+    InfantrySprite *sprite = new InfantrySprite(1.0, m_game->backGnd());
+    sprite-> setDestination(0);             // Sprite immobile
+    soldat = sprite->image(0).scaled(24,24,Qt::KeepAspectRatioByExpanding);
+    delete sprite;
+    
     // load the stopAttackAuto image
-    imageFileName = KGlobal::dirs()->findResource("appdata", skin + "/Images/stopAttackAuto.png");
+    QString imageFileName = KGlobal::dirs()->findResource("appdata", skin + "/Images/stopAttackAuto.png");
     stopAttackAuto.load(imageFileName);
 
     // load the recycle image
@@ -119,7 +122,7 @@ void KRightDialog::displayCountryDetails(const QPointF& countryPoint)
   rightContents.at(1)->setText(i18n("<b>Continent:</b> %1", continent));
   rightContents.at(2)->setText(i18n("<b>Country:</b> %1", pays));
 
-  rightContents.at(3)->setPixmap(soldat.scaled(35,35,Qt::KeepAspectRatioByExpanding));
+  rightContents.at(3)->setPixmap(soldat);
   rightContents.at(6)->setText(units);
   rightContents.at(4)->setText(i18n("<b>Owner:</b> %1", owner));
   rightContents.at(5)->setText(i18n("<b><u>Country details</u></b>"));
@@ -217,20 +220,22 @@ void KRightDialog::displayFightDetails(Country * attaker, Country * defender,int
   flag1->setPixmap(picture1);
   rightContents.at(1)->setText("<i>("+owner_A+")</i> ");
 
-  rightContents.at(2)->setPixmap(soldat.scaled(35,35,Qt::KeepAspectRatioByExpanding));
+  if (!soldat.isNull())
+    rightContents.at(2)->setPixmap(soldat);
   rightContents.at(3)->setText("<b>"+nb_units_A+"</b>");
 
-  rightContents.at(4)->setText(i18n("<font color=\"red\">Attack</font> with %1 armies.<br>", nb_A));
+  rightContents.at(4)->setText(i18np("<font color=\"red\">Attack</font> with 1 army.<br>","<font color=\"red\">Attack</font> with %1 armies.<br>", nb_A));
 
 
   rightContents.at(5)->setText("<u><b>"+i18n(pays_D.toUtf8().data())+"</b></u> ");
   flag2->setPixmap(picture2);
   rightContents.at(6)->setText("<i>("+owner_D+")</i> ");
 
-  rightContents.at(7)->setPixmap(soldat.scaled(35,35,Qt::KeepAspectRatioByExpanding));
+  if (!soldat.isNull())
+    rightContents.at(7)->setPixmap(soldat);
   rightContents.at(8)->setText("<b>"+nb_units_D+"</b> ");
 
-  rightContents.at(9)->setText(i18n("<font color=\"blue\">Defend</font> with %1 armies.<br>", nb_D));
+  rightContents.at(9)->setText(i18np("<font color=\"blue\">Defend</font> with 1 army.<br>","<font color=\"blue\">Defend</font> with %1 armies.<br>", nb_D));
 
   box1->addWidget(rightContents.at(0));
   box1->addWidget(flag1);
@@ -324,7 +329,7 @@ void KRightDialog::displayFightDetails(Country * attaker, Country * defender,int
     rightContents.at(0)->setText("<u><b>"+player->name()+"</b></u> ");
     flag1->setPixmap(player->getFlag()->image(0));
 
-    rightContents.at(1)->setText(i18n("%1 armies to place", nbAvailArmies));
+    rightContents.at(1)->setText(i18np("%1 army to place", "%1 armies to place", nbAvailArmies));
 
     title->addWidget(rightContents.at(0));
     title->addWidget(flag1);
@@ -372,7 +377,7 @@ void KRightDialog::updateRecycleDetails(GameLogic::Country* country, bool recycl
 
   if (recyclePhase)
   {
-    rightContents.at(0)->setText(i18n("<u><b>Change some<br>placements ?</b></u> "));
+    rightContents.at(0)->setText(i18n("<u><b>Change some<br>placements?</b></u> "));
     flag1->hide();
     rightContents.at(1)->setText(QString());
     rightContents.at(2)->setText(QString());
@@ -387,7 +392,7 @@ void KRightDialog::updateRecycleDetails(GameLogic::Country* country, bool recycl
   }
   else
   {
-    rightContents.at(1)->setText(i18n("%1 armies to place", nbAvailArmies));
+    rightContents.at(1)->setText(i18np("1 army to place", "%1 armies to place", nbAvailArmies));
     rightContents.at(2)->setText("<b>"+i18n(country->name().toUtf8().data())+"</b>");
     rightContents.at(3)->setText(i18n("<b>Armies:</b> %1", country->nbArmies()));
     if (nbAvailArmies > 0)
