@@ -33,29 +33,28 @@
 #include <stdio.h>
 #include <time.h>
 
-SudokuBoard::SudokuBoard (QObject * parent,
-                          SudokuType sudokuType, int blockSize)
+SudokuBoard::SudokuBoard (SKGraph * graph)
     :
-    QObject        (parent),
-    m_type         (sudokuType),
-    m_order        (blockSize * blockSize),
-    m_blockSize    (blockSize),
-    m_gridArea     (m_order * m_order),
+    m_graph        (graph),
+    m_type         (graph->specificType()),
+    m_order        (graph->order()),
+    m_blockSize    (graph->base()),
     m_boardSize    (0),
-    m_boardArea    (0),
+    m_boardArea    (graph->size()),
     m_overlap      (0),
-    m_fillStartRow (0),
-    m_fillStartCol (0),
-    m_nGroups      (0),
-    m_groupSize    (0),
-    m_vacant       (0),
-    m_unusable     (-1)
+    m_nGroups      (graph->cliqueCount()),
+    m_groupSize    (m_order),
+    m_vacant       (VACANT),
+    m_unusable     (UNUSABLE)
 {
     dbgLevel = 1;
 
-    m_stats.type      = sudokuType;
-    m_stats.blockSize = blockSize;
+    m_stats.type      = m_type;
+    m_stats.blockSize = m_blockSize;
     m_stats.order     = m_order;
+    m_boardSize       = graph->sizeX(); // TODO - IDW. Rationalise grid sizes.
+    dbe "SudokuBoard constructor: type %d, block %d, order %d, BoardArea %d\n",
+	m_type, m_blockSize, m_order, m_boardArea);
 }
 
 void SudokuBoard::setSeed()
@@ -67,9 +66,7 @@ void SudokuBoard::setSeed()
     }
     else {
         started = true;
-        // m_stats.seed = 95985288; // IDW test.
-        // m_stats.seed = 1317731144; // IDW test.
-        m_stats.seed = time(0); // IDW test.
+        m_stats.seed = time(0);
         qsrand (m_stats.seed);
         dbo "setSeed(): SEED = %d\n", m_stats.seed);
     }
@@ -80,18 +77,23 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
                                   Difficulty difficultyRequired,
                                   Symmetry symmetry)
 {
+    dbo "Entered generatePuzzle()\n");
     QTime t;
     t.start();
     setSeed();
     const int     maxTries = 20;
     int           count = 0;
     float         bestRating = 0.0;
+    int           bestDifficulty = 0;
     int           bestNClues = 0;
     int           bestNGuesses = 0;
     int           bestFirstGuessAt = 0;
     BoardContents currPuzzle;
     BoardContents currSolution;
 
+    if (m_graph->sizeZ() > 1) {
+	symmetry = NONE;		// Symmetry not implemented in 3-D.
+    }
     if (symmetry == RANDOM_SYM) {	// Choose a symmetry at random.
         symmetry = (Symmetry) (qrand() % (int) LAST_CHOICE);
     }
@@ -134,18 +136,41 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
 	// Use the highest rated puzzle so far.
 	if (m_accum.rating > bestRating) {
 	    bestRating       = m_accum.rating;
+	    bestDifficulty   = d;
 	    bestNClues       = m_stats.nClues;
 	    bestNGuesses     = m_accum.nGuesses;
 	    bestFirstGuessAt = m_stats.firstGuessAt;
 	    solution         = currSolution;
 	    puzzle           = currPuzzle;
 	}
+
+	if ((d < difficultyRequired) && (count >= maxTries)) {
+            // Exit after max attempts?
+            QWidget owner;
+            int ans = KMessageBox::questionYesNo (&owner,
+                      i18n("After %1 tries, the best difficulty level achieved "
+			   "is %2, with internal difficulty rating %3, but you "
+			   "requested difficulty level %4. Do you wish to try "
+			   "again or accept the puzzle as is?\n"
+			   "\n"
+			   "If you accept the puzzle, it may help to change to "
+			   "No Symmetry or some low symmetry type, then use "
+			   "Game->New and try generating another puzzle.")
+		      .arg(maxTries).arg(bestDifficulty)
+		      .arg(bestRating, 0, 'f', 1).arg(difficultyRequired),
+                      i18n("Difficulty Level"),
+                      KGuiItem(i18n("&Try Again")), KGuiItem(i18n("&Accept")));
+            if (ans == KMessageBox::Yes) {
+                count = 0;
+                continue;
+            }
+	}
         if ((d >= difficultyRequired) || (count >= maxTries)) {
             QWidget owner;
 	    if (m_accum.nGuesses == 0) {
 		KMessageBox::information (&owner,
-		       i18n("This puzzle can be solved by logic alone. No "
-			    "guessing is required.\n"
+		       i18n("It will be possible to solve the generated puzzle "
+			    "by logic alone. No guessing will be required.\n"
 			    "\n"
 			    "The internal difficulty rating is %1. There are "
 			    "%2 clues at the start and %3 moves to go.")
@@ -156,10 +181,10 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
 	    }
 	    else {
 		KMessageBox::information (&owner,
-		       i18n("This puzzle requires an average of %1 guesses "
-			    "or branch points and if you guess wrong, "
-			    "backtracking is necessary. The first guess comes "
-			    "after %2 moves.\n"
+		       i18n("Solving the generated puzzle will require an "
+			    "average of %1 guesses or branch points and if you "
+			    "guess wrong, backtracking will be necessary. The "
+			    "first guess should come after %2 moves.\n"
 			    "\n"
 			    "The internal difficulty rating is %3, there are "
 			    "%4 clues at the start and %5 moves to go.")
@@ -170,27 +195,8 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
 		       i18n("Difficulty Level"),
 		       "ShowPuzzleStatistics_2");
 	    }
-            if (d >= difficultyRequired) {
-                // Exit when the required difficulty is reached.
-                break;
-            }
-
-            // Exit after max attempts?
-            int ans = KMessageBox::questionYesNo (&owner,
-                      i18n("After %1 tries, the puzzle generator has not "
-                           "reached the difficulty level you requested. Do you "
-                           "wish to try again or accept the puzzle as is?\n"
-			   "\n"
-			   "If you accept the puzzle, it may help to change to "
-			   "No Symmetry or some low symmetry type and then "
-                           "generate another puzzle.").arg(maxTries),
-                      i18n("Difficulty Level"),
-                      KGuiItem(i18n("&Try Again")), KGuiItem(i18n("&Accept")));
-            if (ans == KMessageBox::Yes) {
-                count = 0;
-                continue;
-            }
-            break;
+	    // Exit when the required difficulty or number of tries is reached.
+	    break;
         }
     }
 
@@ -200,10 +206,6 @@ void SudokuBoard::generatePuzzle (BoardContents & puzzle,
         dbo "SOLUTION\n");
         print (solution);
     }
-
-    // TODO - Description of puzzle - seed, date-time, type, blocksize, rating.
-
-    fflush (stdout); // IDW test.
 }
 
 Difficulty SudokuBoard::calculateRating (const BoardContents & puzzle,
@@ -255,22 +257,6 @@ Difficulty SudokuBoard::calculateRating (const BoardContents & puzzle,
     return m_accum.difficulty;
 }
 
-BoardContents & SudokuBoard::fromKSudoku (const QByteArray & values)
-{
-    // SudokuBoard stores by column within row and sets unused cells = -1.
-    // KSudoku stores values by row within column and sets unused cells = 0.
-    clear (m_currentValues);
-    for (int i = 0; i < m_boardSize; i++) {
-        for (int j = 0; j < m_boardSize; j++) {
-            if (m_currentValues.at (i * m_boardSize + j) != m_unusable) {
-                m_currentValues [i * m_boardSize + j] =
-                                ((int) values.at (j * m_boardSize + i));
-            }
-        }
-    }
-    return m_currentValues;
-}
-
 int SudokuBoard::checkPuzzle (const BoardContents & puzzle,
                               const BoardContents & solution)
 {
@@ -382,13 +368,9 @@ BoardContents & SudokuBoard::tryGuesses (GuessingMode gMode = Random)
         guesses = deduceValues (m_currentValues, gMode);
 
         if (guesses.isEmpty()) {
-            // TODO: IDW - Delete the stack somewhere else.  It is needed by
-            //             checkPuzzle() for the multiple-solutions test.
-            // while (! m_states.isEmpty()) {
-                // dbo2 "POP: No more guesses needed at level %d\n",
-                        // m_states.count());
-                // delete m_states.pop();
-            // }
+            // NOTE: We keep the stack of states.  It is needed by checkPuzzle()
+	    //       for the multiple-solutions test and deleted when its parent
+	    //       SudokuBoard object (i.e. this->) is deleted.
             return m_currentValues;
         }
         m_states.push (new State (this, guesses, 0,
@@ -427,9 +409,7 @@ BoardContents SudokuBoard::insertValues (const BoardContents & solution,
             changeClues (puzzle, cell, symmetry, solution);
             changeClues (filled, cell, symmetry, solution);
 
-            dbo2 "CALL deduceValues():\n");
             deduceValues (filled, Random /* NotRandom */);
-            dbo2 "BACK FROM deduceValues():\n");
             if (dbgLevel >= 3) {
                 print (puzzle);
                 print (filled);
@@ -576,19 +556,18 @@ void SudokuBoard::analyseMoves (Statistics & s)
         mType = m_moveTypes.takeFirst();
 	int val = pairVal(m);
 	int pos = pairPos(m);
-	int row = pos / m_boardSize;
-	int col = pos % m_boardSize;
-	int idx = col * m_boardSize + row;	// KSudoku's position-index.
+	int row = m_graph->cellPosY (pos);
+	int col = m_graph->cellPosX (pos);
 
         switch (mType) {
         case Single:
             dbo2 "  Single Pick %d %d row %d col %d\n", val, pos, row+1, col+1);
-	    m_KSudokuMoves.append (idx);
+	    m_KSudokuMoves.append (pos);
             s.nSingles++;
             break;
         case Spot:
             dbo2 "  Single Spot %d %d row %d col %d\n", val, pos, row+1, col+1);
-	    m_KSudokuMoves.append (idx);
+	    m_KSudokuMoves.append (pos);
             s.nSpots++;
             break;
         case Deduce:
@@ -597,7 +576,7 @@ void SudokuBoard::analyseMoves (Statistics & s)
             break;
         case Guess:
             dbo2 "GUESS:        %d %d row %d col %d\n", val, pos, row+1, col+1);
-	    m_KSudokuMoves.append (idx);
+	    m_KSudokuMoves.append (pos);
             if (s.nGuesses < 1) {
                 s.firstGuessAt = s.nSingles + s.nSpots + 1;
             }
@@ -669,14 +648,19 @@ void SudokuBoard::print (const BoardContents & boardValues)
             boardValues.size(), m_boardArea);
         return;
     }
-    for (int i = 0; i < m_boardSize; i++) {
-        if ((i != 0) && (i % m_blockSize == 0)) {
-            printf ("\n");		// Gap between square blocks.
-        }
-        for (int j = 0; j < m_boardSize; j++) {
-            index = i * m_boardSize + j;
+
+    int depth = m_graph->sizeZ();	// If 2-D, depth == 1, else depth > 1.
+    for (int k = 0; k < depth; k++) {
+      int z = (depth > 1) ? (depth - k - 1) : k;
+      for (int j = 0; j < m_graph->sizeY(); j++) {
+	if ((j != 0) && (j % m_blockSize == 0)) {
+	    printf ("\n");		// Gap between square blocks.
+	}
+        int y = (depth > 1) ? (m_graph->sizeY() - j - 1) : j;
+        for (int x = 0; x < m_graph->sizeX(); x++) {
+            index = m_graph->cellIndex (x, y, z);
             value = boardValues.at (index);
-            if (j % m_blockSize == 0) {
+            if (x % m_blockSize == 0) {
                 printf ("  ");		// Gap between square blocks.
             }
             if (value == m_unusable) {
@@ -692,8 +676,9 @@ void SudokuBoard::print (const BoardContents & boardValues)
             }
         }
         printf ("\n");			// End of row.
+      }
+      printf ("\n");			// Next Z or end of 2D puzzle/solution.
     }
-    printf ("\n");			// End of puzzle/solution.
 }
 
 GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
@@ -714,15 +699,15 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
             if (boardValues.at (cell) == m_vacant) {
                 GuessesList newGuesses;
                 qint32 numbers = m_validCellValues.at (cell);
-                // dbo3 ("Cell %d, valid numbers %03o\n", cell, numbers);
+                dbo3 "Cell %d, valid numbers %03o\n", cell, numbers);
                 if (numbers == 0) {
                     dbo2 "SOLUTION FAILED: RETURN at cell %d\n", cell);
                     return solutionFailed (guesses);
                 }
                 int validNumber = 1;
                 while (numbers != 0) {
-                    // dbo3 "Numbers = %03o, validNumber = %d\n",
-                    //         numbers, validNumber);
+                    dbo3 "Numbers = %03o, validNumber = %d\n",
+                            numbers, validNumber);
                     if (numbers & 1) {
                         newGuesses.append (setPair (cell, validNumber));
                     }
@@ -760,8 +745,9 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
         } // Next cell
 
         for (int group = 0; group < m_nGroups; group++) {
+	    QVector<int> cellList = m_graph->clique (group);
             qint32 numbers = m_requiredGroupValues.at (group);
-            // dbo3 "Group %d, valid numbers %03o\n", group, numbers);
+            dbo3 "Group %d, valid numbers %03o\n", group, numbers);
             if (numbers == 0) {
                 continue;
             }
@@ -773,7 +759,7 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
                     GuessesList newGuesses;
                     int index = group * m_groupSize;
                     for (int n = 0; n < m_groupSize; n++) {
-                        cell = m_groupList.at (index);
+			cell = cellList.at (n);
                         if ((m_validCellValues.at (cell) & bit) != 0) {
                             newGuesses.append (setPair (cell, validNumber));
                         }
@@ -855,6 +841,33 @@ GuessesList SudokuBoard::solutionFailed (GuessesList & guesses)
     return guesses;
 }
 
+void SudokuBoard::clear (BoardContents & boardValues)
+{
+    boardValues = m_graph->emptyBoard();	// Set cells vacant or unusable.
+}
+
+BoardContents & SudokuBoard::fillBoard()
+{
+    // Solve the empty board, thus filling it with values at random.  These
+    // values can be the starting point for generating a puzzle and also the 
+    // final solution of that puzzle.
+
+    clear (m_currentValues);
+
+    // Fill a central block with values 1 to m_order in random sequence.  This
+    // reduces the solveBoard() time considerably, esp. if blockSize is 4 or 5.
+    QVector<int> sequence (m_order);
+    QVector<int> cellList = m_graph->clique (m_nGroups / 2);
+    randomSequence (sequence);
+    for (int n = 0; n < m_order; n++) {
+        m_currentValues [cellList.at (n)] = sequence.at (n) + 1;
+    }
+
+    solveBoard (m_currentValues);
+    dbo "BOARD FILLED\n");
+    return m_currentValues;
+}
+
 void SudokuBoard::randomSequence (QVector<int> & sequence)
 {
     if (sequence.isEmpty()) return;
@@ -886,6 +899,7 @@ void SudokuBoard::setUpValueRequirements (BoardContents & boardValues)
     // example 9 bits for a 9x9 grid with 3x3 blocks.
     qint32 allValues = (1 << m_order) - 1;
 
+    dbo2 "Enter setUpValueRequirements()\n");
     if (dbgLevel >= 2) {
         this->print (boardValues);
     }
@@ -898,16 +912,20 @@ void SudokuBoard::setUpValueRequirements (BoardContents & boardValues)
     int    index = 0;
     qint32 bitPattern = 0;
     for (int group = 0; group < m_nGroups; group++) {
+	dbo3 "Group %3d ", group);
+	QVector<int> cellList = m_graph->clique (group);
         bitPattern = 0;
         for (int n = 0; n < m_groupSize; n++) {
-            int value = boardValues.at (m_groupList.at (index)) - 1;
-            if (value >= 0) {
+            int value = boardValues.at (cellList.at (n)) - 1;
+            if (value != m_unusable) {
                 bitPattern |= (1 << value);	// Add bit for each value found.
             }
+	    dbo3 "%3d=%2d ", cellList.at (n), value + 1);
             index++;
         }
         // Reverse all the bits, giving values currently not found in the group.
         m_requiredGroupValues [group] = bitPattern ^ allValues;
+	dbo3 "bits %03o\n", m_requiredGroupValues.at (group));
     }
 
     // Set bit-patterns to show that each cell can accept any value.  Bits are
@@ -930,8 +948,9 @@ void SudokuBoard::setUpValueRequirements (BoardContents & boardValues)
     // can only have 7 or 8, with bit value 192.
     index = 0;
     for (int group = 0; group < m_nGroups; group++) {
+	QVector<int> cellList = m_graph->clique (group);
         for (int n = 0; n < m_order; n++) {
-            int cell  = m_groupList.at (index);
+	    int cell = cellList.at (n);
             m_validCellValues [cell] &= m_requiredGroupValues.at (group);
             index++;
         }   
@@ -961,93 +980,23 @@ void SudokuBoard::updateValueRequirements (BoardContents & boardValues, int cell
     m_validCellValues [cell] = 0;
 
     // Update the requirements for each group to which this cell belongs.
-    int index = m_cellIndex.at (cell);
-    int inmax = m_cellIndex.at (cell + 1);
-    for (int i = index; i < inmax; i++) {
-        int group  = m_cellGroups.at (i);
+    QList<int> groupList = m_graph->cliqueList(cell);
+    foreach (int group, groupList) {
         m_requiredGroupValues [group] &= bitPattern;
 
-        int offset = group * m_groupSize;
+	QVector<int> cellList = m_graph->clique (group);
         for (int n = 0; n < m_order; n++) {
-            int cell  = m_groupList.at (offset + n);
+	    int cell = cellList.at (n);
             m_validCellValues [cell] &= bitPattern;
         }   
     }
 }
 
-qint32 SudokuBoard::indexSquareBlock (qint32 index, int topLeft)
-{
-    int cell      = topLeft;
-    qint32 offset = index;
-    for (int i = 0; i < m_blockSize; i++) {
-        for (int j = 0; j < m_blockSize; j++) {
-            m_groupList [offset] = cell;
-            // dbo3 "Index block [%d,%d], "
-                    // "value %d at offset %d.\n",
-                    // i, j, cell, offset);
-            cell++;
-            offset++;
-        }
-        cell = cell + m_boardSize - m_blockSize;
-    }
-
-    return offset;
-}
-
-void SudokuBoard::indexCellsToGroups()
-{
-    QMultiMap<int, int> cellsToGroups;
-    for (int g = 0; g < m_nGroups; g++) {
-        int offset = g * m_groupSize;
-        for (int n = 0; n < m_groupSize; n++) {
-            cellsToGroups.insert (m_groupList.at (offset + n), g);
-        }
-    }
-
-    m_cellIndex.fill  (0, m_boardArea + 1);
-    m_cellGroups.fill (0, m_nGroups * m_groupSize);
-    int index = 0;
-    for (int cell = 0; cell < m_boardArea; cell++) {
-        m_cellIndex [cell] = index;
-        QList<int> groups = cellsToGroups.values (cell);
-        foreach (int g, groups) {
-            m_cellGroups [index] = g;
-            index++;
-        }
-    }
-    m_cellIndex [m_boardArea] = index;
-
-    dbo3 "indexCellsToGroups():\n");
-    for (int cell = 0; cell < m_boardArea; cell++) {
-        // dbo3 "Cell %3d: groups: ", cell);
-        int index = m_cellIndex.at (cell);
-        int inmax = m_cellIndex.at (cell + 1);
-        for (int n = index; n < inmax; n++) {
-            // dbo3 " %3d", m_cellGroups.at (n));
-        }
-        // dbo3 "\n");
-    }
-}
-
-void SudokuBoard::markUnusable (BoardContents & boardValues,
-                            int imin, int imax, int jmin, int jmax)
-{
-    dbo3 "markUnusable(): Row %2d to %2d, col %2d to %2d\n", // IDW test.
-            imin + 1, imax, jmin + 1, jmax); // IDW test.
-    int index = 0;
-    for (int i = imin; i < imax; i++) {
-        for (int j = jmin; j < jmax; j++) {
-            index = i * m_boardSize + j;
-            boardValues [index] = m_unusable;
-        }
-    }
-}
-
 void SudokuBoard::changeClues (BoardContents & to, int cell, Symmetry type,
                                const BoardContents & from)
 {
     int nSymm = 1;
-    int indices[4];
+    int indices[8];
     nSymm = getSymmetricIndices (m_boardSize, type, cell, indices);
     for (int k = 0; k < nSymm; k++) {
         cell = indices [k];
@@ -1059,22 +1008,26 @@ int SudokuBoard::getSymmetricIndices
                 (int size, Symmetry type, int index, int * out)
 {
     int result = 1;
-    int row    = index / size;
-    int col    = index % size;
     out[0]     = index;
-    bool b[3]  = {1, 1, 1};
+    if (type == NONE) {
+	return result;
+    }
+
+    int row    = m_graph->cellPosY (index);
+    int col    = m_graph->cellPosX (index);
+    int lr     = size - col - 1;		// For left-to-right reflection.
+    int tb     = size - row - 1;		// For top-to-bottom reflection.
 
     switch (type) {
-        case NONE:
-            break;
         case DIAGONAL_1:
 	    // Reflect a copy of the point around two central axes making its
 	    // reflection in the NW-SE diagonal the same as for NE-SW diagonal.
-            row = size - row - 1;
-            col = size - col - 1;
+            row = tb;
+            col = lr;
             // No break; fall through to case DIAGONAL_2.
         case DIAGONAL_2:
-            out[1] = (col * size) + row;	// Reflect in NW-SE diagonal.
+	    // Reflect (col, row) in the main NW-SE diagonal by swapping coords.
+            out[1] = m_graph->cellIndex(row, col);
             result = (out[1] == out[0]) ? 1 : 2;
             break;
         case CENTRAL:
@@ -1082,44 +1035,37 @@ int SudokuBoard::getSymmetricIndices
             result = (out[1] == out[0]) ? 1 : 2;
             break;
 	case SPIRAL:
-	    result = 4;
-            if(size % 2 == 1) {
-		if ((row == col) && (col == (size - 1)/2)) {
-		    result = 1;		// This is the central cell.
-		}
-	    }
-	    if (result == 4) {
-                out[1] = (size - row - 1) * size + size - col - 1;
-                out[2] = col * size + size - row - 1;
-                out[3] = (size - col - 1) * size + row;
+	    if ((size % 2 != 1) || (row != col) || (col != (size - 1)/2)) {
+		result = 4;			// This is not the central cell.
+		out[1] = m_graph->cellIndex(lr,  tb);
+		out[2] = m_graph->cellIndex(row, lr);
+		out[3] = m_graph->cellIndex(tb,  col);
 	    }
             break;
         case FOURWAY:
-            out[1] = out[2] = out[3] = 0;
-            if(size % 2 == 1) {
-                if(col == (size - 1)/2) {
-                    b[0] = b[2] = 0;
-                }
-                if(row == (size - 1)/2) {
-                    b[1] = b[2] = 0;
-                }
-            }
+	    out[1] = m_graph->cellIndex(row, col);	// Interchange X and Y.
+	    out[2] = m_graph->cellIndex(lr,  row);	// Left-to-right.
+	    out[3] = m_graph->cellIndex(row, lr);	// Interchange X and Y.
+	    out[4] = m_graph->cellIndex(col, tb);	// Top-to-bottom.
+	    out[5] = m_graph->cellIndex(tb,  col);	// Interchange X and Y.
+	    out[6] = m_graph->cellIndex(lr,  tb);	// Both L-R and T-B.
+	    out[7] = m_graph->cellIndex(tb,  lr);	// Interchange X and Y.
 
-            if(b[2] == 0) {
-                out[1] = (size - row - 1) * size + size - col - 1;
-                if(out[1] != out[0]) {
-		    result++;
+	    int k;
+	    for (int n = 1; n < 8; n++) {
+		for (k = 0; k < result; k++) {
+		    if (out[n] == out[k]) {
+			break;				// Omit duplicates.
+		    }
 		}
-            }
-            else {
-                out[1] = (size - row - 1) * size + (size - col - 1);
-                out[2] = row * size + (size - col - 1);
-                out[3] = (size - row - 1) * size + col;
-                result = 4;
-            }
+		if (k >= result) {
+		    out[result] = out[n];
+		    result++;				// Use unique positions.
+		}
+	    }
             break;
         case LEFT_RIGHT:
-	    out[1] = row * size + size - col - 1;
+	    out[1] = m_graph->cellIndex(lr,  row);
             result = (out[1] == out[0]) ? 1 : 2;
             break;
         default:
